###################################################################
# GLOBAL HOUSE DIVERSITY PROJECT
# 
# Data extraction code, dimensionality reduction, and  
# statistical analysis  
#
# FROM: Climate, Climate Change and the Global Diversity of Human Houses
#
# Robert R. Dunn, Kathryn R. Kirby, Claire Bowern, Carol R. Ember,
# Russell D. Gray, Joe McCarter, Patrick H. Kavanagh, 
# Michelle Trautwein, Lauren M. Nichols, Michael C. Gavin, and
# Carlos Botero
###################################################################

# load required packages
library(reshape2)
library(jsonlite)
library(tidyverse)
library(fields)
library(psych)

################################################################
# STEP 1 - EXTRACT ENVIRONMENTAL DATA FROM D-PLACE THROUGH GITHUB
# joins environmental data currently split across ~8 folders in D-PLACE repo.
# In each env dataset, there is one row per society.
################################################################
envjson<-jsonlite::fromJSON(url("https://raw.githubusercontent.com/D-PLACE/dplace-data/master/geo/societies_tdwg.json"))
env1<-(data.frame(do.call("rbind", (envjson)))) #note, file should now have 4 variables - lat, code, lon and name [region]
env1<-rownames_to_column(env1,var="soc_id")#make soc_id a column so that env1 can easily be joined with other env data
env1<-as.data.frame(matrix(data=unlist(env1),ncol=5)) #5 columns
env1<-rename(env1, soc_id = V1, env_lat = V2, tdwgRegionCode = V3, env_lon = V4, tdwgRegionName = V5)  #rename these columns, which were added for site functionality; note that lat lon in this dataset has been superceded by lat lon in cultural dataset
env1$env_lat<-as.numeric(as.character(env1$env_lat))
env1$env_lon<-as.numeric(as.character(env1$env_lon))
env1$soc_id<-as.character(env1$soc_id)
env1<-dplyr::select(env1,soc_id,env_lat,env_lon,tdwgRegionCode,tdwgRegionName)

#import remaining env variables using tidyverse 'read_csv'
env2<-read.csv("https://raw.githubusercontent.com/D-PLACE/dplace-data/master/datasets/ecoClimate/data.csv", sep = ",", header = TRUE)
env3<-read.csv("https://raw.githubusercontent.com/D-PLACE/dplace-data/master/datasets/TEOW/data.csv", sep = ",", header = TRUE)   
env4<-read.csv("https://raw.githubusercontent.com/D-PLACE/dplace-data/master/datasets/MODIS/data.csv", sep = ",", header = TRUE)
env5<-read.csv("https://raw.githubusercontent.com/D-PLACE/dplace-data/master/datasets/Kreft/data.csv", sep = ",", header = TRUE)
env6<-read.csv("https://raw.githubusercontent.com/D-PLACE/dplace-data/master/datasets/Jenkins/data.csv", sep = ",", header = TRUE)
env7<-read.csv("https://raw.githubusercontent.com/D-PLACE/dplace-data/master/datasets/GSHHS/data.csv", sep = ",", header = TRUE)     
env8<-read.csv("https://raw.githubusercontent.com/D-PLACE/dplace-data/master/datasets/GMTED2010/data.csv", sep = ",", header = TRUE)

#remove d-place default columns from all 
env2<-dplyr::select(env2, -sub_case, -year, -comment, -references, -source_coded_data, -admin_comment) #all these columns empty
env2$soc_id<-as.character(env2$soc_id)

env3<-dplyr::select(env3, -sub_case, -year, -comment, -references, -source_coded_data, -admin_comment) #all empty except "year", which contains the value "1988" for Biome, and "1981" for EcoRegion
env3$soc_id<-as.character(env3$soc_id)

env4<-dplyr::select(env4, -sub_case, -year, -comment, -references, -source_coded_data, -admin_comment) #all empty except "comment", which contains for 20 societies contains coordinates for nearby location used in data extraction
env4$soc_id<-as.character(env4$soc_id)

env5<-dplyr::select(env5, -sub_case, -year, -comment, -references, -source_coded_data, -admin_comment) #all empty except "year", which contains the value "2007" for all
env5$soc_id<-as.character(env5$soc_id)

env6<-dplyr::select(env6, -sub_case, -year, -comment, -references, -source_coded_data, -admin_comment) #all empty except "year", which contains the value "2013" for all
env6$soc_id<-as.character(env6$soc_id)

env7<-dplyr::select(env7, -sub_case, -year, -comment, -references, -source_coded_data, -admin_comment) #all empty except "comment" for one society (noting manual calculation), and "year" for all ("2015")
env7$soc_id<-as.character(env7$soc_id)

env8<-dplyr::select(env8, -sub_case, -year, -comment, -references, -source_coded_data, -admin_comment) #all these columns empty
env8$soc_id<-as.character(env8$soc_id)

#use tidyverse "spread" to reshape to wide format
env2<-spread(env2, key=var_id, value=code)
env3<-spread(env3, key=var_id, value=code)
env4<-spread(env4, key=var_id, value=code)
env5<-spread(env5, key=var_id, value=code)
env6<-spread(env6, key=var_id, value=code)
env7<-spread(env7, key=var_id, value=code)
env8<-spread(env8, key=var_id, value=code)

#now merge all to create a file with all environmental variables: 
env.all<-left_join(env2,env3)
env.all<-left_join(env.all,env4)
env.all<-left_join(env.all,env5)
env.all<-left_join(env.all,env6)
env.all<-left_join(env.all,env7)
env.all<-left_join(env.all,env8)
env.all<-left_join(env.all,env1, by=c("soc_id")) #add tdwg

rm(env1, env2, env3, env4, env5, env6, env7, env8, envjson)

################################################################
# STEP 2. EXTRACT CULTURAL DATA FROM D-PLACE THROUGH GITHUB
################################################################

#load the EA data csv file from github
cult_data <- read.csv(url("https://raw.githubusercontent.com/D-PLACE/dplace-data/master/datasets/EA/data.csv"), header=TRUE, sep=",")

# reshape stacked cult data, keeping only codes
cult_data <- dplyr::select(cult_data, -sub_case, -year, -comment, -references, -source_coded_data, -admin_comment)
cult_data <- spread(cult_data, key=var_id, value=code)
  
################################################################
# STEP 3 - SELECT AND RECODE CULTURAL VARIABLES USED IN HOUSE ANALYSIS
################################################################
# Select subset of variables of interest:
house.vars <- dplyr::select(cult_data, soc_id,	EA009,	EA030,	EA033,	EA079, EA080, EA081, EA082, EA083)

#Rename numeric variables so more intuitive
house.vars <- rename(house.vars, EA009_polyg = EA009,	EA030_mobil = EA030,	EA033_polit = EA033, 
                     EA079_Hshape = EA079, EA080_Hlevel = EA080, EA081_Hwallm = EA081, 
                     EA082_Hrfshp = EA082, EA083_Hrfmat = EA083)

#Recode variables for analysis

#MODEL PREDICTORS

#EA009_polyg AS CHAR
house.vars$EA009_polygChr = ifelse(house.vars$EA009_polyg %in% c(1,7), "Absent",
                               ifelse(house.vars$EA009_polyg %in% c(2),  "Limited",
                                      ifelse(house.vars$EA009_polyg %in% c(3,4,5,6), "Frequent", NA)))

#EA030_mobil AS CHAR
house.vars$EA030_mobilChr =  ifelse(house.vars$EA030_mobil  %in% c(3,4,5,6,7,8),"Sedentary",
                                ifelse(house.vars$EA030_mobil  %in% c(1,2),"Nomadic",NA))

#EA033_polit - use original coding
house.vars$EA033_politRe =  house.vars$EA033_polit

#HOUSE FEATURES (RESPONSE VARIABLES)

#house.vars$EA079_Hshape AS CHAR
house.vars$EA079_HshapeChr =  ifelse(house.vars$EA079_Hshape %in% c(1,2,3),"RoundedEdges",
                                 ifelse(house.vars$EA079_Hshape %in% c(4,5,6),"AngularEdges",NA))
#house.vars$EA080_Hlevel AS CHAR
house.vars$EA080_HlevelChr =  ifelse(house.vars$EA080_Hlevel %in% c(1),"Subterranean",
                                 ifelse(house.vars$EA080_Hlevel %in% c(2),"Level",
                                        ifelse(house.vars$EA080_Hlevel %in% c(3,4),"Elevated",NA)))

# first need to recode 11 ("wall materials same as roof materials" based on 
# EA083roofmaterials. Note that this creates a new code "0" for snow/ice, 
# which we exclude from our analyses)
#note EA083 code 0 = ice-snow
#note EA083 code 11 = "tile or fired brick", which is coded as "stone, stucco, concrete, fired brick" in EA081
house.vars$EA081_HwallmRe1 =  ifelse(house.vars$EA081_Hwallm  %in% c(11) & house.vars$EA083_Hrfmat %in% c(10),0,
                                 ifelse(house.vars$EA081_Hwallm  %in% c(11) & house.vars$EA083_Hrfmat %in% c(11),1,
                                        ifelse(house.vars$EA081_Hwallm  %in% c(11) & house.vars$EA083_Hrfmat %in% c(1,2,3,4,5,6,7,8,9),house.vars$EA083_Hrfmat,house.vars$EA081_Hwallm)))

#house.vars$EA081_Hwallm AS CHAR
house.vars$EA081_HwallmChr =  ifelse(house.vars$EA081_HwallmRe1 %in% c(0),"IceSnow",
                                 ifelse(house.vars$EA081_HwallmRe1 %in% c(1,2,9),"StoneBrickDaubAdobe",
                                               ifelse(house.vars$EA081_HwallmRe1 %in% c(8),"Thatch",
                                                      ifelse(house.vars$EA081_HwallmRe1 %in% c(5,6,7,10),"HangedFabricsSkinsMatsOpen",
                                                             ifelse(house.vars$EA081_HwallmRe1 %in% c(3,4),"WoodBambooPolesBark",NA)))))

#house.vars$EA082_Hrfshp AS CHAR
house.vars$EA082_HrfshpChr =  ifelse(house.vars$EA082_Hrfshp %in% c(1,2,3,4,5),"RoundedDome",
                                 ifelse(house.vars$EA082_Hrfshp %in% c(6,8,9),"Sloped",
                                        ifelse(house.vars$EA082_Hrfshp %in% c(7),"Flat",NA)))

house.vars<- dplyr::select(house.vars, soc_id, EA009_polygChr, EA030_mobilChr, EA033_politRe, 
                           EA079_HshapeChr, EA080_HlevelChr, EA081_HwallmChr,EA082_HrfshpChr)

# remove unnecessary variables
rm(cult_data)

################################################################
# STEP 4 - PULL IN SOCIETY AND LINGUISTIC METADATA 
################################################################

#load the file linking EA societies to glottocodes 
societies <- read.csv(url("https://raw.githubusercontent.com/D-PLACE/dplace-data/master/datasets/EA/societies.csv"), header=TRUE, sep=",") 
societies <- rename(societies, soc_id = id)

#for now, keep only soc_id and glottocode from the "societies" dataset, we will use this to pull in info on FAMILY from glottolog
soc2<-dplyr::select(societies, soc_id, glottocode)

#Now, pull in family information from Glottolog
temp <- tempfile()
download.file("https://cdstar.shh.mpg.de/bitstreams/EAEA0-D501-DBB8-65C4-0/glottolog_languoid.csv.zip",temp)
glotto <- read.csv(unz(temp, "languoid.csv"), header=TRUE, sep=",")
unlink(temp)

#rename "id" in languoid (glotto) dataset to "glottocode"
names(glotto)
glotto<-rename(glotto, glottocode = id)

#merge the 'societies' and 'glotto' datasets by "glottocode"
soc_glotto<-left_join(soc2, glotto, by = "glottocode") #1291 societies of 16 variables

#adjust family_id so can account for isolates and 'unclassified' langoids in analyses
#for isolates, need to copy language code to 'family'
soc_glotto2<-soc_glotto
soc_glotto2$family_id<-as.character(soc_glotto2$family_id)
soc_glotto2$family_id<-ifelse(soc_glotto2$family_id %in% c(""), soc_glotto2$glottocode, soc_glotto2$family_id)

#merge society, linguistic (glottolog) and cultural data

#first drop glottcode from societies, to avoid duplicate columns
societies2<-dplyr::select(societies, -glottocode)
societies3<-rename(societies2, soc_lat = Lat, soc_lon = Long) 
#second, rename glottolog latitude and longitude to differentiate from society lat lon, and "name" as "lang_name"
soc_glotto2b<-rename(soc_glotto2, glotto_lat = latitude, glotto_lon = longitude, lang_name = name)
#merge society info and linguistic info
soc_glotto3<-left_join(societies3,soc_glotto2b, by = "soc_id")

#keep only most useful columns
soc_glotto4<-dplyr::select(soc_glotto3,soc_id,pref_name_for_society,main_focal_year,soc_lat,soc_lon,glottocode,family_id,glotto_lat,glotto_lon)

#create a dataset for analysis
house.vars<-left_join(soc_glotto4,house.vars, by = "soc_id")

# merge cultural and environmental data
house.vars<-left_join(house.vars,env.all,by=c("soc_id"))

# remove unnecessary variables
rm(glotto, env.all, societies, societies2, societies3, soc2, temp,
   soc_glotto, soc_glotto2, soc_glotto2b, soc_glotto3, soc_glotto4)

################################################################
# STEP 5 - CALCULATE NEIGHBOUR SCORES
# Original script by Patrick Kavanagh
################################################################

# keep only variables likely to be used in analyses, including in PCA
mydata <-
  dplyr::select(
    house.vars,
    soc_id,
    soc_lat,
    soc_lon,
    family_id,
    EA009_polygChr,
    EA030_mobilChr,
    EA033_politRe,
    EA079_HshapeChr,
    EA080_HlevelChr,
    EA081_HwallmChr,
    EA082_HrfshpChr,
    AnnualMeanTemperature,
    AnnualPrecipitationVariance,
    AnnualTemperatureVariance,
    MonthlyMeanPrecipitation,
    PrecipitationPredictability,
    TemperaturePredictability,
    Elevation,
    Slope
  )  

rm(house.vars)

# create a function to compute neighbor scores
ComputeNS <- function(mydata, resp, nbrname) {
  mydata$resp <- mydata[,resp]
  mydata1<-filter(mydata,is.na(resp)==FALSE)
  
  Nneighbors <- 10      #number of neighbours 
  distMatrix <- rdist.earth(mydata1[, c("soc_lon", "soc_lat")], mydata1[, c("soc_lon", "soc_lat")], miles = TRUE, R = NULL)
  
  for ( i in 1:dim(mydata1)[1] ) {
    temp <- sort(distMatrix[i,], index.return=T)$ix[1:(Nneighbors + 1)]
    myNeighbors <- temp[which(temp != i)]
    
    # how common is this society's dependent value among its neighbors?
    mydata1$neighbour.effect[i] <- length(which(as.character(mydata1[myNeighbors,"resp"]) == as.character(mydata1[i,"resp"])))/Nneighbors
  }
  
  # return vector with neighbour effects for that feature
  temp <- dplyr::select(mydata1, soc_id, nbr=neighbour.effect)
  names(temp)[2] <- c(nbrname)
  return(temp)
}

mydata<-left_join(mydata,ComputeNS(mydata, 'EA079_HshapeChr', 'EA079_nbr'), by=c("soc_id"))
mydata<-left_join(mydata,ComputeNS(mydata, 'EA080_HlevelChr', 'EA080_nbr'), by=c("soc_id"))
mydata<-left_join(mydata,ComputeNS(mydata, 'EA081_HwallmChr', 'EA081_nbr'), by=c("soc_id"))
mydata<-left_join(mydata,ComputeNS(mydata, 'EA082_HrfshpChr', 'EA082_nbr'), by=c("soc_id"))

################################################################
# STEP 6 - DIMENSIONALITY REDUCTION IN PREDICTORS VIA PCA
################################################################

# define a function that to find the best scaling components 
# for box-cox transformations
myBCtransform <- function(myvector) {
  require(EnvStats)
  # shift scale to positive numbers and identify optimal lambda for box-cox transformation
  mylambda <- EnvStats::boxcox(as.numeric(myvector)-min(as.numeric(myvector))+1, optimize = T)$lambda
  
  # transform
  myvector <- scale(boxcoxTransform(as.numeric(myvector)-min(as.numeric(myvector))+1, mylambda))
  return (scale(myvector))
}

#create simplified dataset containing only variables for PCA
PCAdata<-data.frame('TransMeanP' = as.numeric(myBCtransform(mydata$MonthlyMeanPrecipitation)), 
                    'TransCVp' = as.numeric(myBCtransform(mydata$AnnualPrecipitationVariance/mydata$MonthlyMeanPrecipitation)), 
                    'TransPp' = as.numeric(myBCtransform(mydata$PrecipitationPredictability)), 
                    'TransMeanT' = as.numeric(myBCtransform(mydata$AnnualMeanTemperature)), 
                    'TransVarT' = as.numeric(myBCtransform(mydata$AnnualTemperatureVariance)), 
                    'TransPt' = as.numeric(myBCtransform(mydata$TemperaturePredictability)), 
                    'TransSlope' = as.numeric(myBCtransform(mydata$Slope)), 
                    'TransElev' = as.numeric(myBCtransform(mydata$Elevation)))

# run initial PCA to determine how many components to keep
require(psych)
myPCA <- pca(PCAdata, nfactors = 4, rotate = "none")
myPCA 

# keep only components with eigenvalues >1 and rotate to facilitate interpretation
myPCA <- pca(PCAdata, nfactors = 3, rotate = "varimax")
myPCA 

mydata$TemperatureHarshness <- -myPCA$scores[,"RC1"] # NB: Invert RC1 axis to facilitate interpretation
mydata$XericHarshness <- -myPCA$scores[,"RC3"] # NB: Invert RC3 axis to facilitate interpretation
mydata$MountainDwelling <- myPCA$scores[,"RC2"]

rm(PCAdata, ComputeNS, myBCtransform)

################################################################
# STEP 7 - BREAK INTO INDIVIDUAL DATASETS FOR ANALYSES
################################################################

#Remove incomplete cases
mydata<-filter(mydata, is.na(EA009_polygChr)==FALSE & is.na(EA030_mobilChr)==FALSE & is.na(EA033_politRe)==FALSE)

#Next create a separate dataset for each feature. 
HouseShape<-mydata %>%
            filter(is.na(EA079_HshapeChr)==FALSE) %>%
            dplyr::select(soc_id, lat = soc_lat, lon = soc_lon, 
                   Polygyny = EA009_polygChr, Mobility = EA030_mobilChr, 
                   PolComplexity = EA033_politRe,
                   LangFamily = family_id, 
                   HouseShape = EA079_HshapeChr, Neighbor = EA079_nbr,
                   TemperatureHarshness, XericHarshness, MountainDwelling)

HouseLevel<-mydata %>%
  filter(is.na(EA080_HlevelChr)==FALSE) %>%
  dplyr::select(soc_id, lat = soc_lat, lon = soc_lon, 
         Polygyny = EA009_polygChr, Mobility = EA030_mobilChr, 
         PolComplexity = EA033_politRe,
         LangFamily = family_id, 
         HouseLevel = EA080_HlevelChr, Neighbor = EA080_nbr,
         TemperatureHarshness, XericHarshness, MountainDwelling)

#Remove 7 societies where wall materials are "IceSnow", as stated in methods.
WallMat<-mydata %>%
  filter(is.na(EA081_HwallmChr)==FALSE & EA081_HwallmChr != "IceSnow") %>%
  dplyr::select(soc_id, lat = soc_lat, lon = soc_lon, 
         Polygyny = EA009_polygChr, Mobility = EA030_mobilChr, 
         PolComplexity = EA033_politRe,
         LangFamily = family_id, 
         WallMatNew = EA081_HwallmChr, Neighbor = EA081_nbr,
         TemperatureHarshness, XericHarshness, MountainDwelling)

RoofShape<-mydata %>%
  filter(is.na(EA082_HrfshpChr)==FALSE) %>%
  dplyr::select(soc_id, lat = soc_lat, lon = soc_lon, 
         Polygyny = EA009_polygChr, Mobility = EA030_mobilChr, 
         PolComplexity = EA033_politRe,
         LangFamily = family_id, 
         RoofShape = EA082_HrfshpChr, Neighbor = EA082_nbr,
         TemperatureHarshness, XericHarshness, MountainDwelling)

################################################################################
################################################################################

################################################################
# STEP 8 - MULTIMODEL INFERENCE
# 
################################################################

require(mclogit)
require(MuMIn)
library(geosphere)

##############################################################
#### COMPANION FUNCTIONS ####
RunMBLOG <- function(dependent, predictors, mydata, WithRandEff = T) {
  mystr <- paste('mymod <- mblogit(formula =', dependent, '~', 
                 paste(predictors, collapse = ' + '))
  if (WithRandEff) {
    mystr <- paste(mystr,', random = ~1|LangFamily, data = mydata, method = "MQL",
                   control = mmclogit.control(maxit = 100))')
  }
  else {
    mystr <- paste(mystr,', data = mydata, method = "MQL", 
                   control = mmclogit.control(maxit = 100))')
  }
  
  eval(parse(text=mystr))
  
  return(mymod)
}

RunAllMods <- function(dependent, predictors, mydata) {
  AllMods <- list(predictors)
  counter <- 2
  for (i in seq((length(predictors)-1), 1, by = -1)) {
    thesemods <- combn(predictors,i)
    for (j in 1:dim(thesemods)[2]){
      AllMods[[counter]] <- thesemods[,j]
      counter <- counter + 1
    }
  }
  
  # estimate all models and compute their AICc weights
  mymods <- list()
  my.aicc <- NULL
  convergences <- NULL
  ModelsThatConverged <- list()
  ModelsThatDidNotConverge <- list()
  
  # start by computing models with random effect
  for ( i in 1:length(AllMods) ) {
    mymods[[i]] <- RunMBLOG(dependent, AllMods[[i]], mydata)
    n <- dim(mydata)[1]
    k <- length(AllMods[[i]]) + 1 # includes intercept as an estimated parameter
    my.aicc[i] <- AIC(mymods[[i]]) + (2*k^2 + 2*k)/(n-k-1)
    convergences[i] <- mymods[[i]]$converged
    if (mymods[[i]]$converged) { 
      ModelsThatConverged <- c(ModelsThatConverged, mymods[[i]]$formula) 
    }
    else { 
      ModelsThatDidNotConverge <- c(ModelsThatDidNotConverge, mymods[[i]]$formula) 
    }
    
    print(paste(length(ModelsThatConverged), 'out of', i, "(Total:", length(AllMods),")"))
  }
  
  # now a model with ONLY the random effect
  mymods[[length(AllMods) + 1]] <- RunMBLOG(dependent, '1', mydata)
  n <- dim(mydata)[1]
  k <- 1
  my.aicc[length(AllMods) + 1] <- AIC(mymods[[length(AllMods) + 1]]) + (2*k^2 + 2*k)/(n-k-1)
  convergences[length(AllMods) + 1] <- mymods[[length(AllMods) + 1]]$converged
  if (mymods[[length(AllMods) + 1]]$converged) { 
    ModelsThatConverged <- c(ModelsThatConverged, mymods[[length(AllMods) + 1]]$formula) 
  }
  else { 
    ModelsThatDidNotConverge <- c(ModelsThatDidNotConverge, mymods[[length(AllMods) + 1]]$formula) 
  }
  
  # now compute models without random effect
  for ( i in 1:length(AllMods) ) {
    mymods[[length(AllMods) + 1 + i]] <- RunMBLOG(dependent, AllMods[[i]],
                                                  mydata, WithRandEff = F)
    n <- dim(mydata)[1]
    k <- length(AllMods[[i]]) + 1
    my.aicc[length(AllMods) + 1 + i] <- AIC(mymods[[length(AllMods) + 1 + i]]) + (2*k^2 + 2*k)/(n-k-1)
    convergences[length(AllMods) + 1 + i] <- mymods[[length(AllMods) + 1 + i]]$converged
    
    if (mymods[[length(AllMods) + 1 + i]]$converged) { 
      ModelsThatConverged <- c(ModelsThatConverged, mymods[[length(AllMods) + 1 + i]]$formula) 
    }
    else { 
      ModelsThatDidNotConverge <- c(ModelsThatDidNotConverge, mymods[[length(AllMods) + 1 + i]]$formula) 
    }
  }
  
  # get corresponding AICc weights (only consider models that converged)...
  my.waicc <- Weights(my.aicc[convergences])
  
  # now we can compute relative variable importance (NB: preliminary explorations indicate that
  # the full model converges every time so we can start with this)
  RVI <- data.frame(matrix(0,length(predictors)+2,1))
  row.names(RVI) <- c('(Intercept)',predictors,'LangFamily')
  names(RVI) <- 'RVI'
  RVI['(Intercept)',1] <- 1
  RVI['LangFamily',1] <- sum(my.waicc[1:(length(AllMods) + 1)])
  
  mods.converged <- which(convergences)
  for (i in 1:length(my.waicc)) {
    for (j in 1:length(predictors)) {
      if (length(grep(predictors[j], mymods[[mods.converged[i]]]$formula))>0) {
        RVI[predictors[j],1] <- RVI[predictors[j],1] + my.waicc[i]
      }
    }
  }
  
  # ... and we can estimate the model average
  
  # start with the weighted value of estimates in the fully parameterized model
  my.mod.avg.estimate <- summary(mymods[[1]])$coefficients[,'Estimate']*my.waicc[1]
  my.mod.avg.SE <- summary(mymods[[1]])$coefficients[,'Std. Error']*my.waicc[1]
  
  # now go one by one adding weighted values of parameters that are 
  # present in each model (only count models that converged)
  for (i in 2:length(mods.converged)) {
    for (j in 1:length(mymods[[mods.converged[i]]]$coefficients)) {
      myindex <- which(names(my.mod.avg.estimate)==names(mymods[[mods.converged[i]]]$coefficient)[j])
      my.mod.avg.estimate[myindex] <- my.mod.avg.estimate[myindex] + 
        summary(mymods[[mods.converged[i]]])$coefficients[j,'Estimate']*my.waicc[i]
      my.mod.avg.SE[myindex] <- my.mod.avg.SE[myindex] + 
        summary(mymods[[mods.converged[i]]])$coefficients[j,'Std. Error']*my.waicc[i]
    }
  }
  
  return(list("allmods" = mymods, "Coefficients" = my.mod.avg.estimate, "SE" = my.mod.avg.SE,
              "RVI" = round(RVI*100)/100, "ModelsThatConverged" = ModelsThatConverged,
              'ModelsThatDidNotConverge' = ModelsThatDidNotConverge))
}

#### END COMPANION FUNCTIONS ####
##############################################################

# define fully parameterized model
mypreds <- c('Neighbor', 'Polygyny', 'Mobility', 
             'XericHarshness', 'MountainDwelling',
             'PolComplexity', 'TemperatureHarshness', 
             'PolComplexity:TemperatureHarshness')

#######
# House level
HouseLevel$soc_id <- as.factor(HouseLevel$soc_id)
HouseLevel$Polygyny <- as.factor(HouseLevel$Polygyny)
HouseLevel$Mobility <- as.factor(HouseLevel$Mobility)
HouseLevel$LangFamily <- as.factor(HouseLevel$LangFamily)
HouseLevel$HouseLevel <- as.factor(HouseLevel$HouseLevel)

# explore fully parameterized model for evidence of spatial autocorrelation
mymod <- mblogit(formula = HouseLevel ~ Neighbor + Polygyny + 
                   Mobility + PolComplexity*TemperatureHarshness +
                   XericHarshness + MountainDwelling,
                 random = ~1|LangFamily, method = 'MQL', data = HouseLevel, 
                 control = mmclogit.control(maxit = 30))

# notice the summary gives you n-1 sets of estimates (each describes one binary regression
# comparing the baseline category in the dependent against a given other category)
summary(mymod)

# compute distance matrix 
dist.coords <- distm(HouseLevel[, c("lon", "lat")])

# OK, let's look at evidence for spatial autocorrelation in residuals
# (NB: Our model residuals are extracted for each equation calculating the probability
# of a given response, i.e., you get N x obs residuals, where N is the number of categories
# in the response variable. Let's check for evidence of spatial autocorrelation in any of the
# response categories)
for ( i in 0:(length(levels(HouseLevel$HouseLevel))-1) ) { # plot correlograms for each probability equation
  my.correlogram <- spatialRF::moran(
    x = mymod$working.residuals[(1:dim(HouseLevel)[1])+i*(dim(HouseLevel)[1])],
    distance.matrix = dist.coords
  )
  print(my.correlogram$test)
}

# NO NEED for correction for spatial autocorrelation

# run all models (note that some combinations of parameters will not converge 
# due to complete separation. We do not consider those in our analyses)
mymod.avg.HouseLevel <- RunAllMods('HouseLevel', mypreds, HouseLevel)

# Model average values
cbind('Estimate' = round(mymod.avg.HouseLevel$Coefficients*1000)/1000, 
      'Lower C.I' = round((mymod.avg.HouseLevel$Coefficients - 1.96*mymod.avg.HouseLevel$SE)*1000)/1000,
      'Upper C.I' = round((mymod.avg.HouseLevel$Coefficients + 1.96*mymod.avg.HouseLevel$SE)*1000)/1000,
      'Relative variable importance' = round(mymod.avg.HouseLevel$RVI*100)/100)

# list of models that converged
mymod.avg.HouseLevel$ModelsThatConverged

######################################
HouseShape$soc_id <- as.factor(HouseShape$soc_id)
HouseShape$Polygyny <- as.factor(HouseShape$Polygyny)
HouseShape$Mobility <- as.factor(HouseShape$Mobility)
HouseShape$LangFamily <- as.factor(HouseShape$LangFamily)
HouseShape$HouseShape <- as.factor(HouseShape$HouseShape)

# explore fully parameterized model for evidence of spatial autocorrelation
mymod <- mblogit(formula = HouseShape ~ Neighbor + Polygyny + 
                   Mobility + PolComplexity*TemperatureHarshness +
                   XericHarshness + MountainDwelling,
                 random = ~1|LangFamily, method = 'MQL', data = HouseShape, 
                 control = mmclogit.control(maxit = 30))

# notice the summary gives you n-1 sets of estimates (each describes one binary regression
# comparing the baseline category in the dependent against a given other category)
summary(mymod)

# compute distance matrix 
dist.coords <- distm(HouseShape[, c("lon", "lat")])

# OK, let's look at evidence for spatial autocorrelation in residuals
# (NB: Our model residuals are extracted for each equation calculating the probability
# of a given response, i.e., you get N x obs residuals, where N is the number of categories
# in the response variable. Let's check for evidence of spatial autocorrelation in any of the
# response categories)
for ( i in 0:(length(levels(HouseShape$HouseShape))-1) ) { # plot correlograms for each probability equation
  my.correlogram <- spatialRF::moran(
    x = mymod$working.residuals[(1:dim(HouseShape)[1])+i*(dim(HouseShape)[1])],
    distance.matrix = dist.coords
  )
  print(my.correlogram$test)
} # NO NEED for correction for spatial autocorrelation

# run all models (note that some combinations of parameters will not converge 
# due to complete separation. We do not consider those in our analyses)
mymod.avg.HouseShape <- RunAllMods('HouseShape', mypreds, HouseShape)

# Model average values
cbind('Estimate' = round(mymod.avg.HouseShape$Coefficients*1000)/1000, 
      'Lower C.I' = round((mymod.avg.HouseShape$Coefficients - 1.96*mymod.avg.HouseShape$SE)*1000)/1000,
      'Upper C.I' = round((mymod.avg.HouseShape$Coefficients + 1.96*mymod.avg.HouseShape$SE)*1000)/1000,
      'Relative variable importance' = round(mymod.avg.HouseShape$RVI*100)/100)

# list of models that converged
mymod.avg.HouseShape$ModelsThatConverged

######################################
RoofShape$LangFamily <- as.factor(RoofShape$LangFamily)

# this model has problems of complete or quasi-complete separation so we try eliminating rare languages
RoofShape <- RoofShape[RoofShape$LangFamily %in% names(which(summary(RoofShape$LangFamily)>25)), ]
RoofShape$LangFamily <- as.factor(as.character(RoofShape$LangFamily))
RoofShape$soc_id <- as.factor(RoofShape$soc_id)
RoofShape$Polygyny <- as.factor(RoofShape$Polygyny)
RoofShape$Mobility <- as.factor(RoofShape$Mobility)
RoofShape$RoofShape <- as.factor(RoofShape$RoofShape)

# explore fully parameterized model for evidence of spatial autocorrelation
mymod <- mblogit(formula = RoofShape ~ Neighbor + Polygyny + 
                   Mobility + PolComplexity*TemperatureHarshness +
                   XericHarshness + MountainDwelling,
                 random = ~1|LangFamily, method = 'MQL', data = RoofShape, 
                 control = mmclogit.control(maxit = 100))

# notice the summary gives you n-1 sets of estimates (each describes one binary regression
# comparing the baseline category in the dependent against a given other category)
summary(mymod)

# compute distance matrix 
dist.coords <- distm(RoofShape[, c("lon", "lat")])

# OK, let's look at evidence for spatial autocorrelation in residuals
# (NB: Our model residuals are extracted for each equation calculating the probability
# of a given response, i.e., you get N x obs residuals, where N is the number of categories
# in the response variable. Let's check for evidence of spatial autocorrelation in any of the
# response categories)
for ( i in 0:(length(levels(RoofShape$RoofShape))-1) ) { # plot correlograms for each probability equation
  my.correlogram <- spatialRF::moran(
    x = mymod$working.residuals[(1:dim(RoofShape)[1])+i*(dim(RoofShape)[1])],
    distance.matrix = dist.coords
  )
  print(my.correlogram$test)
} # NO NEED for correction for spatial autocorrelation

# run all models (note that some combinations of parameters will not converge 
# due to complete separation. We do not consider those in our analyses)
mymod.avg.RoofShape <- RunAllMods('RoofShape', mypreds, RoofShape)

# Model average values
cbind('Estimate' = round(mymod.avg.RoofShape$Coefficients*1000)/1000, 
      'Lower C.I' = round((mymod.avg.RoofShape$Coefficients - 1.96*mymod.avg.RoofShape$SE)*1000)/1000,
      'Upper C.I' = round((mymod.avg.RoofShape$Coefficients + 1.96*mymod.avg.RoofShape$SE)*1000)/1000,
      'Relative variable importance' = round(mymod.avg.RoofShape$RVI*100)/100)

# list of models that converged
mymod.avg.RoofShape$ModelsThatConverged

######################################
WallMat$soc_id <- as.factor(WallMat$soc_id)
WallMat$Polygyny <- as.factor(WallMat$Polygyny)
WallMat$Mobility <- as.factor(WallMat$Mobility)
WallMat$LangFamily <- as.factor(WallMat$LangFamily)
WallMat$WallMat <- as.factor(WallMat$WallMat)

# explore fully parameterized model for evidence of spatial autocorrelation
mymod <- mblogit(formula = WallMat ~ Neighbor + Polygyny + 
                   Mobility + PolComplexity*TemperatureHarshness +
                   XericHarshness + MountainDwelling,
                 random = ~1|LangFamily, method = 'MQL', data = WallMat, 
                 control = mmclogit.control(maxit = 30))

# notice the summary gives you n-1 sets of estimates (each describes one binary regression
# comparing the baseline category in the dependent against a given other category)
summary(mymod)

# compute distance matrix 
dist.coords <- distm(WallMat[, c("lon", "lat")])

# OK, let's look at evidence for spatial autocorrelation in residuals
# (NB: Our model residuals are extracted for each equation calculating the probability
# of a given response, i.e., you get N x obs residuals, where N is the number of categories
# in the response variable. Let's check for evidence of spatial autocorrelation in any of the
# response categories)
for ( i in 0:(length(levels(WallMat$WallMat))-1) ) { # plot correlograms for each probability equation
  my.correlogram <- spatialRF::moran(
    x = mymod$working.residuals[(1:dim(WallMat)[1])+i*(dim(WallMat)[1])],
    distance.matrix = dist.coords
  )
  print(my.correlogram$test)
} # NO NEED for correction for spatial autocorrelation

# run all models (note that some combinations of parameters will not converge 
# due to complete separation. We do not consider those in our analyses)
mymod.avg.WallMat <- RunAllMods('WallMat', mypreds, WallMat)

# Model average values
cbind('Estimate' = round(mymod.avg.WallMat$Coefficients*1000)/1000, 
      'Lower C.I' = round((mymod.avg.WallMat$Coefficients - 1.96*mymod.avg.WallMat$SE)*1000)/1000,
      'Upper C.I' = round((mymod.avg.WallMat$Coefficients + 1.96*mymod.avg.WallMat$SE)*1000)/1000,
      'Relative variable importance' = round(mymod.avg.WallMat$RVI*100)/100)

# list of models that converged
mymod.avg.WallMat$ModelsThatConverged

##################################################################
# BUILD TABLES FOR OUTPUT

# 1) table for relative variable importance
mytable.RVI <- data.frame(cbind("Ground plan" = mymod.avg.HouseShape$RVI, "Floor level" = mymod.avg.HouseLevel$RVI, 
                                "Wall material" = mymod.avg.WallMat$RVI, "Roof shape" = mymod.avg.RoofShape$RVI))

mytable.RVI <- rbind("Chance prediction" = round(c(1/length(levels(as.factor(HouseShape$HouseShape))), 1/length(levels(as.factor(HouseLevel$HouseLevel))),
                                                   1/length(levels(as.factor(WallMat$WallMat))), 1/length(levels(as.factor(RoofShape$RoofShape))))*100)/100, 
                     "Most common category" = c(
                       max(round(summary(as.factor(HouseShape$HouseShape))/(sum(summary(as.factor(HouseShape$HouseShape))))*100)/100),
                       max(round(summary(as.factor(HouseLevel$HouseLevel))/(sum(summary(as.factor(HouseLevel$HouseLevel))))*100)/100),
                       max(round(summary(as.factor(WallMat$WallMat))/(sum(summary(as.factor(WallMat$WallMat))))*100)/100),
                       max(round(summary(as.factor(RoofShape$RoofShape))/(sum(summary(as.factor(RoofShape$RoofShape))))*100)/100)),
                     "Predictive value of average model" = NA, mytable.RVI)

colnames(mytable.RVI) <- c("Ground plan", "Floor level", "Wall material", "Roof shape")

# compute Predictive values
BuildEquation <- function (Category, mymod.avg, Neighbor, 
                           Polygyny, Mobility, PolComplexity, 
                           XericHarshness, MountainDwelling, TemperatureHarshness) {
  
  myval <- mymod.avg$Coefficients[paste0(Category,'~(Intercept)')] + 
    mymod.avg$Coefficients[paste0(Category,'~Neighbor')]*Neighbor + 
    mymod.avg$Coefficients[paste0(Category,'~XericHarshness')]*XericHarshness +
    mymod.avg$Coefficients[paste0(Category,'~MountainDwelling')]*MountainDwelling +
    mymod.avg$Coefficients[paste0(Category,'~PolComplexity')]*PolComplexity + 
    mymod.avg$Coefficients[paste0(Category,'~TemperatureHarshness')]*TemperatureHarshness + 
    mymod.avg$Coefficients[paste0(Category,'~PolComplexity:TemperatureHarshness')]*PolComplexity*TemperatureHarshness
  
  if (Polygyny=="Frequent") {
    myval <- myval + mymod.avg$Coefficients[paste0(Category,'~PolygynyFrequent')]
  }
  if (Polygyny=="Limited") {
    myval <- myval + mymod.avg$Coefficients[paste0(Category,'~PolygynyLimited')]
  }
  
  if (Mobility=="Sedentary") {
    myval <- myval + mymod.avg$Coefficients[paste0(Category,'~MobilitySedentary')]
  }
  return(exp(myval))
}

# Ground plan
pctcorrect <- 0
for(i in 1:dim(HouseShape)[1]) {
  myval <- BuildEquation('RoundedEdges', mymod.avg.HouseShape, HouseShape$Neighbor[i], 
                         HouseShape$Polygyny[i], HouseShape$Mobility[i], 
                         HouseShape$PolComplexity[i], HouseShape$XericHarshness[i], 
                         HouseShape$MountainDwelling[i], HouseShape$TemperatureHarshness[i])
  
  if (myval>0.5) {
    if (HouseShape$HouseShape[i]=="RoundedEdges") {
      pctcorrect <- pctcorrect + 1/dim(HouseShape)[1]
    }
  }
  else {  
    if (HouseShape$HouseShape[i]!="RoundedEdges") {
      pctcorrect <- pctcorrect + 1/dim(HouseShape)[1]
    }
  }
}

mytable.RVI['Predictive value of average model','Ground plan'] <- pctcorrect

# FLOOR LEVEL
pctcorrect <- 0
for(i in 1:dim(HouseLevel)[1]) {
  myval <- 1
  myval <- c(myval, BuildEquation('Level', mymod.avg.HouseLevel, HouseLevel$Neighbor[i], 
                                  HouseLevel$Polygyny[i], HouseLevel$Mobility[i], 
                                  HouseLevel$PolComplexity[i], HouseLevel$XericHarshness[i], 
                                  HouseLevel$MountainDwelling[i], HouseLevel$TemperatureHarshness[i]))
  
  myval <- c(myval, BuildEquation('Subterranean', mymod.avg.HouseLevel, HouseLevel$Neighbor[i], 
                                  HouseLevel$Polygyny[i], HouseLevel$Mobility[i], 
                                  HouseLevel$PolComplexity[i], HouseLevel$XericHarshness[i], 
                                  HouseLevel$MountainDwelling[i], HouseLevel$TemperatureHarshness[i]))
  
  myval <- myval / sum(myval)
  
  if (which(myval==max(myval))==1) {
    if (HouseLevel$HouseLevel[i]=="Elevated") {
      pctcorrect <- pctcorrect + 1/dim(HouseLevel)[1]
    }
  }
  else {  
    if (which(myval==max(myval))==2) {
      if (HouseLevel$HouseLevel[i]=="Level") {
        pctcorrect <- pctcorrect + 1/dim(HouseLevel)[1]
      }
    }
    else {
      if (HouseLevel$HouseLevel[i]=="Subterranean") {
        pctcorrect <- pctcorrect + 1/dim(HouseLevel)[1]
      }
    }
  }
}

mytable.RVI['Predictive value of average model','Floor level'] <- pctcorrect

# WALL MATERIAL
pctcorrect <- 0
for(i in 1:dim(WallMat)[1]) {
  myval <- 1
  myval <- c(myval, BuildEquation('StoneBrickDaubAdobe', mymod.avg.WallMat, WallMat$Neighbor[i],
                                  WallMat$Polygyny[i], WallMat$Mobility[i], 
                                  WallMat$PolComplexity[i], WallMat$XericHarshness[i], 
                                  WallMat$MountainDwelling[i], WallMat$TemperatureHarshness[i]))
  
  myval <- c(myval, BuildEquation('Thatch', mymod.avg.WallMat, WallMat$Neighbor[i], 
                                  WallMat$Polygyny[i], WallMat$Mobility[i], 
                                  WallMat$PolComplexity[i], WallMat$XericHarshness[i], 
                                  WallMat$MountainDwelling[i], WallMat$TemperatureHarshness[i]))
  
  myval <- c(myval, BuildEquation('WoodBambooPolesBark', mymod.avg.WallMat, WallMat$Neighbor[i], 
                                  WallMat$Polygyny[i], WallMat$Mobility[i], 
                                  WallMat$PolComplexity[i], WallMat$XericHarshness[i], 
                                  WallMat$MountainDwelling[i], WallMat$TemperatureHarshness[i]))
  
  myval <- myval / sum(myval)
  
  
  if (which(myval==max(myval))==1) {
    if (WallMat$WallMat[i]=="HangedFabricsSkinsMatsOpen") {
      pctcorrect <- pctcorrect + 1/dim(WallMat)[1]
    }
  }
  else {  
    if (which(myval==max(myval))==2) {
      if (WallMat$WallMat[i]=="StoneBrickDaubAdobe") {
        pctcorrect <- pctcorrect + 1/dim(WallMat)[1]
      }
    }
    else {
      if (which(myval==max(myval))==3) {
        if (WallMat$WallMat[i]=="Thatch") {
          pctcorrect <- pctcorrect + 1/dim(WallMat)[1]
        }
      }
      else {
        if (WallMat$WallMat[i]=="WoodBambooPolesBark") {
          pctcorrect <- pctcorrect + 1/dim(WallMat)[1]
        }
      }
    }
  }
}

mytable.RVI['Predictive value of average model','Wall material'] <- pctcorrect

# ROOF SHAPE
pctcorrect <- 0
for(i in 1:dim(RoofShape)[1]) {
  myval <- 1
  myval <- c(myval, BuildEquation('RoundedDome', mymod.avg.RoofShape, RoofShape$Neighbor[i], 
                                  RoofShape$Polygyny[i], RoofShape$Mobility[i], 
                                  RoofShape$PolComplexity[i], RoofShape$XericHarshness[i], 
                                  RoofShape$MountainDwelling[i], RoofShape$TemperatureHarshness[i]))
  
  myval <- c(myval, BuildEquation('Sloped', mymod.avg.RoofShape, RoofShape$Neighbor[i], 
                                  RoofShape$Polygyny[i], RoofShape$Mobility[i], 
                                  RoofShape$PolComplexity[i], RoofShape$XericHarshness[i], 
                                  RoofShape$MountainDwelling[i], RoofShape$TemperatureHarshness[i]))
  myval <- myval / sum(myval)
  
  if (which(myval==max(myval))==1) {
    if (RoofShape$RoofShape[i]=="Flat") {
      pctcorrect <- pctcorrect + 1/dim(RoofShape)[1]
    }
  }
  else {  
    if (which(myval==max(myval))==2) {
      if (RoofShape$RoofShape[i]=="RoundedDome") {
        pctcorrect <- pctcorrect + 1/dim(RoofShape)[1]
      }
    }
    else {
      if (RoofShape$RoofShape[i]=="Sloped") {
        pctcorrect <- pctcorrect + 1/dim(RoofShape)[1]
      }
    }
  }
}

mytable.RVI['Predictive value of average model','Roof shape'] <- pctcorrect

# simplify output
mytable.RVI <- round(mytable.RVI*100)/100

#######################################
#######################################
#######################################
# tables for coefficients 

# Ground plan (House shape)
mytable.HS <- data.frame("Predictor"= names(mymod.avg.HouseShape$Coefficients),
                         "Posterior distribution" = NA)

mytable.HS$Posterior.distribution <- paste((round(mymod.avg.HouseShape$Coefficients*100))/100, 
                                           "±", (round(mymod.avg.HouseShape$SE*100)/100))
summary(as.factor(HouseShape$HouseShape))

#######################
# House level
mytable.HL <- data.frame("Predictor"= names(mymod.avg.HouseLevel$Coefficients),
                         "Posterior distribution" = NA)

mytable.HL$Posterior.distribution <- paste((round(mymod.avg.HouseLevel$Coefficients*100))/100, 
                                           "±", (round(mymod.avg.HouseLevel$SE*100)/100))
summary(as.factor(HouseLevel$HouseLevel))

#######################
# Roof shape
mytable.RS <- data.frame("Predictor"= names(mymod.avg.RoofShape$Coefficients),
                         "Posterior distribution" = NA)

mytable.RS$Posterior.distribution <- paste((round(mymod.avg.RoofShape$Coefficients*100))/100, 
                                           "±", (round(mymod.avg.RoofShape$SE*100)/100))
summary(as.factor(RoofShape$RoofShape))

#######################
# Wall material
mytable.WM <- data.frame("Predictor"= names(mymod.avg.WallMat$Coefficients),
                         "Posterior distribution" = NA)

mytable.WM$Posterior.distribution <- paste((round(mymod.avg.WallMat$Coefficients*100))/100, 
                                           "±", (round(mymod.avg.WallMat$SE*100)/100))
summary(as.factor(WallMat$WallMat))

###################################################

